/*
  event.h
  Definitions needed for event programming

  Copyright (c) 1995-98 Dragos Acostachioaie, Bernd Kalbfuss

  This file is part of Xterminal library.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the license, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this program; if not, write to the Free
  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _XEVENT_H
#define _XEVENT_H

#pragma interface

#include <xbclass.h>

#undef min
#undef max
#undef erase
#undef clear
#undef _G_HAVE_BOOL

#include <deque.h>
#include <multimap.h>

// Standard Event Classes
#define EV_NOTHING   0x0000
#define EV_KEYBOARD  0x0002
#define EV_MOUSE     0x0004
#define EV_SIGNAL    0x0100
#define EV_MESSAGE   0x0200
#define EV_COMMAND   0x0400
#define EV_BROADCAST 0x0800

/**
    @memo Basic event class
    @author Dragos Acostachioaie
    @version 0.1
*/  
class XEvent {
 public:

  /// Object event is addressed
  unsigned long Id;

  /// Event class
  unsigned Class;

  /// Event type
  unsigned Type;
};

// Mouse Event Types
#define EV_MOUSEDOWN 0x0001
#define EV_MOUSEUP   0x0002
#define EV_MOUSEMOVE 0x0003

/**
    @memo Mouse event class
    @author Dragos Acostachioaie
    @version 0.1
*/  
class XMouseEvent : public XEvent {
 public:

  /// Constructs a mouse event
  XMouseEvent();

  /// Button number
  unsigned Button;

  /// Number of clicks
  unsigned Clicks;

  /// Position of mouse pointer at the time event was generated
  XPoint Position;
};

// Keyboard Event Types
#define EV_KEYDOWN   0x0001

/**
    @memo Keyboard event class
    @author Dragos Acostachioaie
    @version 0.1
*/  
class XKeyboardEvent : public XEvent {
 public:

  /// Constructs a keyboard event
  XKeyboardEvent();

  /// Key code
  unsigned KeyCode;

  /// Key code if it's a simple character
  unsigned CharCode;
};

// Message Classes
#define MC_CLIENT_TO_SERVER 0x01
#define MC_CLIENT_TO_CLIENT 0x02
#define MC_SERVER_TO_CLIENT 0x03
#define MC_BROADCAST      0x04

// Basically Messages
#define MT_ANSWER_REQUEST  0x0000
#define MT_REQUEST_HONORED 0x0001
#define MT_RELEASE_FOCUS   0x0002
#define MT_RESUME_FOCUS    0x0003

/**
    @memo Message event class
    @author Dragos Acostachioaie
    @version 0.1
*/  
class XMessage : public XEvent {
 public:

  /// Constructs a message
  XMessage();

  /// Object that sent the message
  unsigned long SenderId;

  /// a "long" message
  long Message;

  /// a "void" message, so we can send pointers between objects
  void *Body;

  /// size of the "void" message
  unsigned long Size;
};

/**
    @memo Event listener registration
    @author Bernd Kalbfuss
    @version 0.1	
*/
class XEventBroker
{
 public:

  /** Registers an event listener for the specified event class.
      @param __Listener object id of listener 
      @param __Class event class
  */
  void AddListener(unsigned long __Listener, unsigned __Class);

  void AddListener(unsigned long __Listener);

  /** Removes the event listener for the specified event class.
      @param __Listener object id of listener 
      @param __Class event class
  */
  void DelListener(unsigned long __Listener, unsigned __Class);

  /** Removes the specified event listener for all event classes.
      @param __Listener object id of listener
  */
  void DelListener(unsigned long __Listener);

  /// Handles an incoming event and distributes it to all registered listeners.
  virtual void HandleEvent(XEvent *__Event);

  /** Sends a "long" message.
      @param __Receiver destination object id
      @param __Class message class
      @param __Message the message itself 
      @return 0 if receiver does not exist
  */
  int SendMessage(unsigned long __Receiver, unsigned long __Class, long __Message);

  /** Sends a "void *" message.
      @param __Receiver destination object id
      @param __Class message class
      @param __Body the message itself
      @param __Size size of __Body pointer
      @return 0 if receiver does not exist */
  int SendMessage(unsigned long __Receiver, unsigned long __Class, void *__Body, unsigned long __Size);

 protected:
  typedef multimap < unsigned, unsigned long, less < const unsigned > > Map;
  typedef pair < const unsigned, unsigned long > KeyValPair;

  /// map for binding listeners to events
  Map __map;
};

/**
    @memo Listener for registration at broker
    @author Bernd Kalbfuss
    @version 0.1
*/
class XEventListener
{
 public:

  /** Process all events distributed from the broker. Derived
      listeners have to implement this method.
  */
  virtual int ProcessEvent(XEvent *__Event) = 0;
};

/** Clears the event passed as parameter.
    In fact, sets the Class field to EV_NOTHING, without deleting it.
*/
void ClearEvent(XEvent *__Ev);

/// Returns the first available event, zero if none
XEvent *FetchEvent();

/// Returns the first available event for object __Id, zero if none
XEvent *FetchEvent(unsigned long __Id);

void InitEvents();
void DoneEvents();

void _InitEventQueue();
void _PutEvent(XEvent *__Ev);
void _PutEventInFront(XEvent *__Ev);
XEvent *_GetEvent();
XEvent *_GetFirstEvent(XEvent *__Ev);
int _EventQueueIsEmpty();
void _ClearEventQueue();

// Standard Command Codes
#define CM_QUIT      0x0001
#define CM_MENU      0x0002
#define CM_CLOSE     0x0004
#define CM_OK        0x000A
#define CM_CANCEL    0x000B

#endif
