/*
  xobject.h
  Definitions for XObject and objects table

  Copyright (c) 1995-98 Dragos Acostachioaie, Bernd Kalfbuss

  This file is part of Xterminal library.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the license, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this program; if not, write to the Free
  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _XOBJECT_H
#define _XOBJECT_H

#pragma interface

#include <xevent.h>

#undef min
#undef max
#undef erase
#undef clear
#undef _G_HAVE_BOOL

#include <SLList.h>
#include <multimap.h>

/**
    @memo Listener for registration at broker
    @author Bernd Kalbfuss
    @version 0.1
*/
class XObjectEventListener : public XEventListener
{
 public:

  /** Receives incoming events and distributes them between the following
      processing methods.
      These methods should be overloaded for active objects, of course only for
      event classes the respective object cares. If the object treats the event,
      these functions must returns true, otherwise false. That's because
      Xterminal can't figure out all the times which object an event is for.
      Instead, events are passed to all registered listeners and then objects
      are watched to see which is responding to it (an example of such event is
      an mouse event). Once the event gets processed, the remaining listeners
      does not receive it any more.
  */
  virtual int ProcessEvent(XEvent *__Event);

  /** Process a mouse event (EV_MOUSE).
      Be careful with mouse events because you must check if the mouse pointer
      is inside object's bounds, otherwise you must return false!
  */
  virtual int ProcessMouseEvent(XMouseEvent *__Event);

  /// Process a keyboard event (EV_KEYBOARD)
  virtual int ProcessKeyboardEvent(XKeyboardEvent *__Event);

  /// Process a message (EV_MESSAGE)
  virtual int ProcessMessage(XMessage *__Event);

  /// Process a command (EV_COMMAND)
  virtual int ProcessCommand(XMessage *__Event);

  /// Process a signal (EV_SIGNAL)
  virtual int ProcessSignal(XEvent *__Event);

  /// Process a broadcast event (EV_BROADCAST)
  virtual int ProcessBroadcast(XEvent *__Event);
};

/**
    @memo Foundation class
    @author Dragos Acostachioaie
    @version 0.1
*/  
class XObject : public XEventBroker, public XObjectEventListener {
 public:

  /// Constructs a new XObject with the specified bounds
  XObject(XRect __Bounds);

  /// Destructs the object instance
  virtual ~XObject();

  /** Computes the object's size and the real bounds.
      This method is used by the RegisterClient method
  */
  virtual void CalculateBounds();

  /** Paints the visual object content onto the terminal's screen.
      Remember to call XRefresh() function before return so the draw becomes
      visible!
  */
  virtual void Draw();

  /// Returns the object's bounds
  XRect GetBounds();

  /// Returns the first event addressed to this object, zero if none
  virtual XEvent *GetEvent();

  /// Returns the object's id
  unsigned long GetId();

  /// Returns the objects's class name
  char *GetObjClass();

  /// Returns the object's origin
  XPoint GetOrigin();

  /// Returns the object's attached color palette
  virtual char *GetPalette();

  /// Returns the color value with __Index position inside the palette
  virtual unsigned GetPalEntry(unsigned __Index);

  /// Returns the object's size
  XPoint GetSize();

  /** Returns the object state register.
      The returned value must be processed next with the state masks the
      object uses to gain valuable information
  */
  unsigned long GetState();

  /** Returns the value of the specified object's state
      @param __Mask state mask
      @return non-zero if bit set
  */
  virtual unsigned GetState(unsigned long __Mask);

  /// Handles the event passed as argument, by calling ProcessEvent() method
  virtual void HandleEvent(XEvent *__Ev);

  /// Moves the object's origin to the specified point
  virtual void MoveTo(XPoint __p);

  /** Receives incoming events and distributes them using the ProcessEvent()
      method of XObjectEventListener class, also being careful not to treat
      keyboard events if the object does not own the focus
  */
  virtual int ProcessEvent(XEvent *__Event);

  /** Registers the specified XObject as client.
      It also enable the SM_NO_REFRESH flag for the newly registered client
  */
  virtual void RegisterClient(XObject *__NewClient);

  /** Sets the object's attached color palette
      @param __Pal new palette
      @param __PalLen new palette size
  */
  virtual void SetPalette(char *__Pal, unsigned __PalLen);

  /** Sets the value of the specified object state
      @param __Mask state mask
      @param __BoolValue set bit if non-zero, otherwise clear it
  */
  virtual void SetState(unsigned long __Mask, unsigned __BoolValue);

 protected:

  /// Object's unique identificator
  unsigned long Id;

  /** Current properties of the object.
      The different states can be ORed. The common used states are:
      <p>- SM_VISIBLE indicates that the object is currently visible on the
           terminal's screen, so it is in a drawn state
      <p>- SM_CURSOR_VISIBLE indicates that the cursor is currently visible and
           it's inside the object's bounds
      <p>- SM_FOCUSED indicates that the object has the control at the
	   respective moment (only one object can have the control at a
	   moment, so if the user press, let's say, a key, this object will
	   receive the event)
      <p>- SM_DRAGGING indicates that the objects is currently dragged on the
           terminal's screen
      <p>- SM_DISABLED indicates that the object is deactivated at the moment
      <p>- SM_ACTIVE indicates that the object is active, so the user has
	   control over it
      <p>- SM_NO_REFRESH indicates that the object's Draw() method will not dump
           it's contents to the screen by calling XRefresh(), but will let it's
	   server to do this instead
  */
  unsigned long State;

  /// Object's upper-left coordinate, relative to it's server
  XPoint Origin;

  /// Object's size
  XPoint Size;

  /// Cursor's position inside the object's bounds
  XPoint Cursor;

  /// This are the *real* bounds, in absolute coordinates
  XRect Bounds;

  /// Name of the class the object belongs, choosen by the programmer
  char ObjClass[40];

  /// Object's attached color palette
  char Pal[80];

  /// Palette size
  unsigned PalLen;

  /** Server to which the object is registered (and which services he uses),
      or zero if the object have no server.
      (If you are not familiar with the client-server concept, try to think
      to the server as the object's parent)
  */
  XObject *Server;

  /// Linked list of object's registered clients
  SLList < XObject * > Clients;

  /** Sends a "long" message
      @param __Receiver destination object id
      @param __Class message class
      @param __Message the message itself
  */
  int SendMessage(unsigned long __Receiver, unsigned long __Class, long __Message);

  /** Sends a "void *" message
      @param __Receiver destination object id
      @param __Class message class
      @param __Body the message itself
      @param __Size size of __Body pointer
  */
  int SendMessage(unsigned long __Receiver, unsigned long __Class, void *__Body, unsigned long __Size);

 private:
  void InsertClient(XObject *__Client);
  void DeleteClient(XObject *__Client);
};

// XObject State Masks
#define SM_VISIBLE		0x0001
#define SM_CURSOR_VISIBLE	0x0002
#define SM_FOCUSED		0x0004
#define SM_DRAGGING		0x0008
#define SM_DISABLED		0x0010
#define SM_EXPOSED		0x0020
#define SM_ACTIVE		0x0040
#define SM_NO_REFRESH		0x0080

// Object Classes
#define OC_UNKNOWN       "Unkown"
#define OC_XOBJECT       "XObject"

#define RESERVED_IDS	0x0f
#define MAX_OBJECTS	0xffffffff

int IsObjectRegistered(unsigned long __Id);
int IsObjectRegistered(XObject *__Ptr);
XObject* GetOPtr(unsigned long __Id);
unsigned long GetOId(XObject *__Ptr);

unsigned long _InsertObject(XObject *__Ptr);
void _DeleteObject(unsigned long __Id);
void _DeliverMessage(XMessage *__Msg);

#endif
